@@ -4,6 +4,7 @@ namespace Icinga\Module\Director\Controllers;
 
 use Exception;
 use gipfl\Web\Widget\Hint;
+use Icinga\Module\Director\Data\Exporter;
 use Icinga\Module\Director\Db\Branch\Branch;
 use Icinga\Module\Director\Forms\ImportRowModifierForm;
 use Icinga\Module\Director\Forms\ImportSourceForm;
@@ -84,7 +85,7 @@ class ImportsourceController extends ActionController
         $this->addMainActions();
         $source = $this->getImportSource();
         if ($this->params->get('format') === 'json') {
-            $this->sendJson($this->getResponse(), $source->export());
+            $this->sendJson($this->getResponse(), (new Exporter($this->db()))->export($source));
             return;
         }
         $this->addTitle(
@@ -179,9 +180,7 @@ class ImportsourceController extends ActionController
                 'target' => '_blank',
                 'class'  => 'icon-download',
             ]
-        ))->add(Link::create('[..]', '#', null, [
-            'onclick' => 'javascript:$("table.raw-data-table").toggleClass("collapsed");'
-        ]));
+        ));
         try {
             (new ImportsourceHookTable())->setImportSource($source)->renderTo($this);
         } catch (Exception $e) {
