@@ -2,12 +2,12 @@
 
 namespace Icinga\Module\Director\Cli;
 
+use gipfl\Json\JsonDecodeException;
 use gipfl\Json\JsonString;
 use Icinga\Cli\Command as CliCommand;
 use Icinga\Module\Director\Application\MemoryLimit;
 use Icinga\Module\Director\Core\CoreApi;
 use Icinga\Module\Director\Db;
-use Icinga\Module\Director\Exception\JsonException;
 use Icinga\Module\Director\Objects\IcingaEndpoint;
 use Icinga\Application\Config;
 use RuntimeException;
@@ -31,15 +31,17 @@ class Command extends CliCommand
      */
     protected function parseJson($json)
     {
-        $res = json_decode($json);
-
-        if ($res === null) {
-            $this->fail('Invalid JSON: %s', $this->getLastJsonError());
+        try {
+            return JsonString::decode($json);
+        } catch (JsonDecodeException $e) {
+            $this->fail('Invalid JSON: %s', $e->getMessage());
         }
-
-        return $res;
     }
 
+    /**
+     * @param string $msg
+     * @return never-return
+     */
     public function fail($msg)
     {
         $args = func_get_args();
@@ -47,16 +49,8 @@ class Command extends CliCommand
         if (count($args)) {
             $msg = vsprintf($msg, $args);
         }
-
-        throw new RuntimeException($msg);
-    }
-
-    /**
-     * @return string
-     */
-    protected function getLastJsonError()
-    {
-        return JsonException::getJsonErrorMessage(json_last_error());
+        echo $this->screen->colorize("ERROR", 'red') . ": $msg\n";
+        exit(1);
     }
 
     /**
@@ -88,9 +82,9 @@ class Command extends CliCommand
     {
         MemoryLimit::raiseTo('1024M');
 
-        ini_set('max_execution_time', 0);
+        ini_set('max_execution_time', '0');
         if (version_compare(PHP_VERSION, '7.0.0') < 0) {
-            ini_set('zend.enable_gc', 0);
+            ini_set('zend.enable_gc', '0');
         }
 
         return $this;
