@@ -2,6 +2,8 @@
 
 namespace Icinga\Module\Director\Dashboard\Dashlet;
 
+use Icinga\Module\Director\Auth\Permission;
+
 class CheckCommandsDashlet extends Dashlet
 {
     protected $icon = 'wrench';
@@ -21,7 +23,7 @@ class CheckCommandsDashlet extends Dashlet
 
     public function listRequiredPermissions()
     {
-        return array('director/admin');
+        return [Permission::ADMIN];
     }
 
     public function getUrl()
