@@ -2,6 +2,8 @@
 
 namespace Icinga\Module\Director\Dashboard\Dashlet;
 
+use Icinga\Module\Director\Auth\Permission;
+
 abstract class ChoicesDashlet extends Dashlet
 {
     protected $icon = 'flapping';
@@ -14,7 +16,7 @@ abstract class ChoicesDashlet extends Dashlet
     public function getSummary()
     {
         return $this->translate(
-            'Combine multiple templates into meaningful Choices, making live'
+            'Combine multiple templates into meaningful Choices, making life'
             . ' easier for your users'
         );
     }
@@ -36,6 +38,6 @@ abstract class ChoicesDashlet extends Dashlet
 
     public function listRequiredPermissions()
     {
-        return array('director/admin');
+        return [Permission::ADMIN];
     }
 }
