@@ -2,7 +2,11 @@
 
 namespace Icinga\Module\Director\DirectorObject\Automation;
 
-use Icinga\Module\Director\Core\Json;
+use gipfl\Json\JsonDecodeException;
+use gipfl\Json\JsonEncodeException;
+use gipfl\Json\JsonString;
+use Icinga\Module\Director\Data\Exporter;
+use Icinga\Module\Director\Data\ObjectImporter;
 use Icinga\Module\Director\Db;
 use Icinga\Module\Director\Data\Db\DbObject;
 use Icinga\Module\Director\Objects\DirectorDatafield;
@@ -26,7 +30,9 @@ use Icinga\Module\Director\Objects\IcingaUserGroup;
 use Icinga\Module\Director\Objects\ImportSource;
 use Icinga\Module\Director\Objects\SyncRule;
 use InvalidArgumentException;
+use Ramsey\Uuid\UuidInterface;
 use RuntimeException;
+use stdClass;
 
 class BasketSnapshot extends DbObject
 {
@@ -214,16 +220,11 @@ class BasketSnapshot extends DbObject
 
     /**
      * @param Db $connection
-     * @param bool $replace
      * @throws \Icinga\Exception\NotFoundError
      */
-    public function restoreTo(Db $connection, $replace = true)
+    public function restoreTo(Db $connection)
     {
-        static::restoreJson(
-            $this->getJsonDump(),
-            $connection,
-            $replace
-        );
+        static::restoreJson($this->getJsonDump(), $connection);
     }
 
     /**
@@ -237,61 +238,49 @@ class BasketSnapshot extends DbObject
             'basket_uuid' => $basket->get('uuid')
         ]);
         $snapshot->objects = [];
-        foreach ((array) Json::decode($string) as $type => $objects) {
+        foreach ((array) JsonString::decode($string) as $type => $objects) {
             $snapshot->objects[$type] = (array) $objects;
         }
 
         return $snapshot;
     }
 
-    public static function restoreJson($string, Db $connection, $replace = true)
+    public static function restoreJson($string, Db $connection)
     {
-        $snapshot = new static();
-        $snapshot->restoreObjects(
-            Json::decode($string),
-            $connection,
-            $replace
-        );
+        (new static())->restoreObjects(JsonString::decode($string), $connection);
     }
 
     /**
-     * @param $all
-     * @param Db $connection
-     * @param bool $replace
      * @throws \Icinga\Module\Director\Exception\DuplicateKeyException
      * @throws \Zend_Db_Adapter_Exception
      * @throws \Icinga\Exception\NotFoundError
+     * @throws JsonDecodeException
      */
-    protected function restoreObjects($all, Db $connection, $replace = true)
+    protected function restoreObjects(stdClass $all, Db $connection)
     {
         $db = $connection->getDbAdapter();
         $db->beginTransaction();
         $fieldResolver = new BasketSnapshotFieldResolver($all, $connection);
-        $this->restoreType($all, 'DataList', $fieldResolver, $connection, $replace);
-        $this->restoreType($all, 'DatafieldCategory', $fieldResolver, $connection, $replace);
+        $this->restoreType($all, 'DataList', $fieldResolver, $connection);
+        $this->restoreType($all, 'DatafieldCategory', $fieldResolver, $connection);
         $fieldResolver->storeNewFields();
         foreach ($this->restoreOrder as $typeName) {
-            $this->restoreType($all, $typeName, $fieldResolver, $connection, $replace);
+            $this->restoreType($all, $typeName, $fieldResolver, $connection);
         }
         $db->commit();
     }
 
     /**
-     * @param $all
-     * @param $typeName
-     * @param BasketSnapshotFieldResolver $fieldResolver
-     * @param Db $connection
-     * @param $replace
      * @throws \Icinga\Exception\NotFoundError
      * @throws \Icinga\Module\Director\Exception\DuplicateKeyException
      * @throws \Zend_Db_Adapter_Exception
+     * @throws JsonDecodeException
      */
     public function restoreType(
-        &$all,
-        $typeName,
+        stdClass $all,
+        string $typeName,
         BasketSnapshotFieldResolver $fieldResolver,
-        Db $connection,
-        $replace
+        Db $connection
     ) {
         if (isset($all->$typeName)) {
             $objects = (array) $all->$typeName;
@@ -299,11 +288,10 @@ class BasketSnapshot extends DbObject
             return;
         }
         $class = static::getClassForType($typeName);
-
+        $importer = new ObjectImporter($connection);
         $changed = [];
-        foreach ($objects as $key => $object) {
-            /** @var DbObject $new */
-            $new = $class::import($object, $connection, $replace);
+        foreach ($objects as $object) {
+            $new = $importer->import($class, $object);
             if ($new->hasBeenModified()) {
                 if ($new instanceof IcingaObject && $new->supportsImports()) {
                     /** @var ExportInterface $new */
@@ -322,7 +310,6 @@ class BasketSnapshot extends DbObject
                     $fieldResolver->relinkObjectFields($new, $object);
                 }
             }
-            $allObjects[spl_object_hash($new)] = $object;
         }
 
         /** @var IcingaObject $object */
@@ -331,7 +318,7 @@ class BasketSnapshot extends DbObject
         }
         foreach ($changed as $key => $new) {
             // Store related fields. As objects might have formerly been
-            // un-stored, let's to it right here
+            // un-stored, let's do it right here
             if ($new instanceof IcingaObject) {
                 $fieldResolver->relinkObjectFields($new, $objects[$key]);
             }
@@ -355,10 +342,9 @@ class BasketSnapshot extends DbObject
     }
 
     /**
-     * @return BasketContent
      * @throws \Icinga\Exception\NotFoundError
      */
-    protected function getContent()
+    protected function getContent(): BasketContent
     {
         if ($this->content === null) {
             $this->content = BasketContent::load($this->get('content_checksum'), $this->getConnection());
@@ -377,26 +363,25 @@ class BasketSnapshot extends DbObject
     }
 
     /**
-     * @return string
-     * @throws \Icinga\Exception\NotFoundError
+     * @throws \Icinga\Exception\NotFoundError|JsonEncodeException
      */
-    public function getJsonSummary()
+    public function getJsonSummary(): string
     {
         if ($this->hasBeenLoadedFromDb()) {
             return $this->getContent()->get('summary');
         }
 
-        return Json::encode($this->getSummary(), JSON_PRETTY_PRINT);
+        return JsonString::encode($this->getSummary(), JSON_PRETTY_PRINT);
     }
 
     /**
      * @return array|mixed
-     * @throws \Icinga\Exception\NotFoundError
+     * @throws \Icinga\Exception\NotFoundError|JsonDecodeException
      */
     public function getSummary()
     {
         if ($this->hasBeenLoadedFromDb()) {
-            return Json::decode($this->getContent()->get('summary'));
+            return JsonString::decode($this->getContent()->get('summary'));
         }
 
         $summary = [];
@@ -409,7 +394,7 @@ class BasketSnapshot extends DbObject
 
     /**
      * @return string
-     * @throws \Icinga\Exception\NotFoundError
+     * @throws \Icinga\Exception\NotFoundError|JsonEncodeException
      */
     public function getJsonDump()
     {
@@ -417,16 +402,40 @@ class BasketSnapshot extends DbObject
             return $this->getContent()->get('content');
         }
 
-        return Json::encode($this->objects, JSON_PRETTY_PRINT);
+        try {
+            return JsonString::encode($this->objects, JSON_PRETTY_PRINT);
+        } catch (JsonEncodeException $e) {
+            foreach ($this->objects as $type => $objects) {
+                foreach ($objects as $object) {
+                    try {
+                        JsonString::encode($object);
+                    } catch (JsonEncodeException $singleError) {
+                        $dump = var_export($object, true);
+                        if (function_exists('iconv')) {
+                            $dump = iconv('UTF-8', 'UTF-8//IGNORE', $dump);
+                        }
+                        throw new JsonEncodeException(sprintf(
+                            'Failed to encode object ot type "%s": %s, %s',
+                            $type,
+                            $dump,
+                            $singleError->getMessage()
+                        ), $singleError->getCode());
+                    }
+                }
+            }
+
+            throw $e;
+        }
     }
 
     protected function addAll($typeName)
     {
         list($class, $filter) = static::getClassAndObjectTypeForType($typeName);
+        $connection = $this->getConnection();
+        assert($connection instanceof Db);
 
         /** @var IcingaObject $dummy */
         $dummy = $class::create();
-        /** @var ExportInterface $object */
         if ($dummy instanceof IcingaObject && $dummy->supportsImports()) {
             $db = $this->getDb();
             $select = $db->select()->from($dummy->getTableName());
@@ -441,12 +450,13 @@ class BasketSnapshot extends DbObject
             ) {
                 $select->where('object_type = ?', 'template');
             }
-            $all = $class::loadAll($this->getConnection(), $select);
+            $all = $class::loadAll($connection, $select);
         } else {
-            $all = $class::loadAll($this->getConnection());
+            $all = $class::loadAll($connection);
         }
+        $exporter = new Exporter($connection);
         foreach ($all as $object) {
-            $this->objects[$typeName][$object->getUniqueIdentifier()] = $object->export();
+            $this->objects[$typeName][$object->getUniqueIdentifier()] = $exporter->export($object);
         }
     }
 
@@ -457,29 +467,36 @@ class BasketSnapshot extends DbObject
         }
     }
 
+    /**
+     * @return ExportInterface|DbObject|null
+     */
+    public static function instanceByUuid(string $typeName, UuidInterface $uuid, Db $connection)
+    {
+        /** @var class-string<DbObject> $class */
+        $class = static::getClassForType($typeName);
+        /** @var ExportInterface $object */
+        return $class::loadWithUniqueId($uuid, $connection);
+    }
+
     /**
      * @param $typeName
      * @param $identifier
      * @param Db $connection
-     * @return ExportInterface|null
+     * @return ExportInterface|DbObject|null
      */
     public static function instanceByIdentifier($typeName, $identifier, Db $connection)
     {
+        /** @var class-string<DbObject> $class */
         $class = static::getClassForType($typeName);
-        if (substr($class, -13) === 'IcingaService') {
+        if ($class === IcingaService::class) {
             $identifier = [
                 'object_type' => 'template',
                 'object_name' => $identifier,
             ];
         }
-        /** @var ExportInterface $object */
-        if ($class::exists($identifier, $connection)) {
-            $object = $class::load($identifier, $connection);
-        } else {
-            $object = null;
-        }
 
-        return $object;
+        /** @var ExportInterface $object */
+        return $class::loadOptional($identifier, $connection);
     }
 
     /**
@@ -490,13 +507,14 @@ class BasketSnapshot extends DbObject
     {
         /** @var Db $connection */
         $connection = $this->getConnection();
+        $exporter = new Exporter($connection);
         $object = static::instanceByIdentifier(
             $typeName,
             $identifier,
             $connection
         );
         if ($object !== null) {
-            $this->objects[$typeName][$identifier] = $object->export();
+            $this->objects[$typeName][$identifier] = $exporter->export($object);
         }
     }
 }
