@@ -183,6 +183,10 @@ class IcingaObjectGroups implements Iterator, Countable, IcingaConfigRenderer
             return $this;
         }
 
+        if (is_int($group)) {
+            $group = (string) $group;
+        }
+
         /** @var IcingaObjectGroup $class */
         $class = $this->getGroupClass();
 
@@ -224,7 +228,7 @@ class IcingaObjectGroups implements Iterator, Countable, IcingaConfigRenderer
         } else {
             throw new RuntimeException(
                 'Invalid group object: %s',
-                var_export($group, 1)
+                var_export($group, true)
             );
         }
 
