@@ -28,17 +28,28 @@ class IcingaObjectMultiRelations implements Iterator, Countable, IcingaConfigRen
 
     protected $relationIdColumn;
 
+    protected $relatedShortName;
+
+    protected $legacyPropertyName;
+
     private $position = 0;
 
     private $db;
 
     protected $idx = array();
 
-    public function __construct(IcingaObject $object, $propertyName, $relatedObjectClass)
+    public function __construct(IcingaObject $object, $propertyName, $config)
     {
         $this->object = $object;
         $this->propertyName = $propertyName;
-        $this->relatedObjectClass = $relatedObjectClass;
+
+        if (is_object($config) || is_array($config)) {
+            foreach ($config as $k => $v) {
+                $this->$k = $v;
+            }
+        } else {
+            $this->relatedObjectClass = $config;
+        }
     }
 
     public function getObjects()
@@ -46,11 +57,13 @@ class IcingaObjectMultiRelations implements Iterator, Countable, IcingaConfigRen
         return $this->relations;
     }
 
+    #[\ReturnTypeWillChange]
     public function count()
     {
         return count($this->relations);
     }
 
+    #[\ReturnTypeWillChange]
     public function rewind()
     {
         $this->position = 0;
@@ -61,6 +74,7 @@ class IcingaObjectMultiRelations implements Iterator, Countable, IcingaConfigRen
         return $this->modified;
     }
 
+    #[\ReturnTypeWillChange]
     public function current()
     {
         if (! $this->valid()) {
@@ -70,16 +84,19 @@ class IcingaObjectMultiRelations implements Iterator, Countable, IcingaConfigRen
         return $this->relations[$this->idx[$this->position]];
     }
 
+    #[\ReturnTypeWillChange]
     public function key()
     {
         return $this->idx[$this->position];
     }
 
+    #[\ReturnTypeWillChange]
     public function next()
     {
         ++$this->position;
     }
 
+    #[\ReturnTypeWillChange]
     public function valid()
     {
         return array_key_exists($this->position, $this->idx);
@@ -222,7 +239,7 @@ class IcingaObjectMultiRelations implements Iterator, Countable, IcingaConfigRen
         } else {
             throw new ProgrammingError(
                 'Invalid related object: %s',
-                var_export($relation, 1)
+                var_export($relation, true)
             );
         }
 
@@ -238,15 +255,26 @@ class IcingaObjectMultiRelations implements Iterator, Countable, IcingaConfigRen
         return $this->propertyName;
     }
 
+    protected function getRelatedShortName()
+    {
+        if ($this->relatedShortName === null) {
+            /** @var IcingaObject $class */
+            $class = $this->getRelatedClassName();
+            $this->relatedShortName = $class::create()->getShortTableName();
+        }
+
+        return $this->relatedShortName;
+    }
+
     protected function getTableName()
     {
-        $class = $this->getRelatedClassName();
-        return $this->object->getTableName() . '_' . $class::create()->getShortTableName();
+        return $this->object->getTableName() . '_' . $this->getRelatedShortName();
     }
 
     protected function getRelatedTableName()
     {
         if ($this->relatedTableName === null) {
+            /** @var IcingaObject $class */
             $class = $this->getRelatedClassName();
             $this->relatedTableName = $class::create()->getTableName();
         }
@@ -257,8 +285,7 @@ class IcingaObjectMultiRelations implements Iterator, Countable, IcingaConfigRen
     protected function getRelationIdColumn()
     {
         if ($this->relationIdColumn === null) {
-            $class = $this->getRelatedClassName();
-            $this->relationIdColumn = $class::create()->getShortTableName();
+            $this->relationIdColumn = $this->getRelatedShortName();
         }
 
         return $this->relationIdColumn;
@@ -302,7 +329,7 @@ class IcingaObjectMultiRelations implements Iterator, Countable, IcingaConfigRen
 
         $class = $this->getRelatedClassName();
         $this->relations = $class::loadAll($connection, $query, 'object_name');
-        $this->cloneStored();
+        $this->setBeingLoadedFromDb();
 
         return $this;
     }
@@ -310,13 +337,10 @@ class IcingaObjectMultiRelations implements Iterator, Countable, IcingaConfigRen
     public function store()
     {
         $db = $this->getDb();
-
         $stored = array_keys($this->stored);
         $relations = array_keys($this->relations);
 
         $objectId = $this->object->id;
-        $type = $this->getType();
-
         $type = $this->getType();
         $objectCol = $type . '_id';
         $relationCol = $this->getRelationIdColumn() . '_id';
@@ -348,12 +372,12 @@ class IcingaObjectMultiRelations implements Iterator, Countable, IcingaConfigRen
                 )
             );
         }
-        $this->cloneStored();
+        $this->setBeingLoadedFromDb();
 
         return true;
     }
 
-    protected function cloneStored()
+    public function setBeingLoadedFromDb()
     {
         $this->stored = array();
         foreach ($this->relations as $k => $v) {
@@ -420,6 +444,11 @@ class IcingaObjectMultiRelations implements Iterator, Countable, IcingaConfigRen
             return '';
         }
 
-        return c1::renderKeyValue($this->propertyName, c1::renderArray($relations));
+        if ($this->legacyPropertyName === null) {
+            return '    # not supported in legacy: ' .
+                c1::renderKeyValue($this->propertyName, c1::renderArray($relations), '');
+        }
+
+        return c1::renderKeyValue($this->legacyPropertyName, c1::renderArray($relations));
     }
 }
