@@ -3,12 +3,14 @@
 namespace Icinga\Module\Director\Objects;
 
 use Icinga\Application\Benchmark;
+use Icinga\Data\Filter\Filter;
 use Icinga\Exception\NotFoundError;
 use Icinga\Module\Director\Application\MemoryLimit;
 use Icinga\Module\Director\Data\Db\DbObjectWithSettings;
 use Icinga\Module\Director\Db;
 use Icinga\Module\Director\DirectorObject\Automation\ExportInterface;
 use Icinga\Module\Director\Exception\DuplicateKeyException;
+use Icinga\Module\Director\Filter\FilterEnrichment;
 use Icinga\Module\Director\Hook\PropertyModifierHook;
 use Icinga\Module\Director\Import\Import;
 use Icinga\Module\Director\Import\SyncUtils;
@@ -52,74 +54,13 @@ class ImportSource extends DbObjectWithSettings implements ExportInterface
 
     private $newRowModifiers;
 
-    /**
-     * @return \stdClass
-     */
-    public function export()
-    {
-        $plain = $this->getProperties();
-        $plain['originalId'] = $plain['id'];
-        unset($plain['id']);
-
-        foreach ($this->stateProperties as $key) {
-            unset($plain[$key]);
-        }
-
-        $plain['settings'] = (object) $this->getSettings();
-        $plain['modifiers'] = $this->exportRowModifiers();
-        ksort($plain);
-
-        return (object) $plain;
-    }
-
-    /**
-     * @param $plain
-     * @param Db $db
-     * @param bool $replace
-     * @return ImportSource
-     * @throws DuplicateKeyException
-     * @throws NotFoundError
-     */
-    public static function import($plain, Db $db, $replace = false)
-    {
-        $properties = (array) $plain;
-        if (isset($properties['originalId'])) {
-            $id = $properties['originalId'];
-            unset($properties['originalId']);
-        } else {
-            $id = null;
-        }
-        $name = $properties['source_name'];
-
-        if ($replace && static::existsWithNameAndId($name, $id, $db)) {
-            $object = static::loadWithAutoIncId($id, $db);
-        } elseif ($replace && static::exists($name, $db)) {
-            $object = static::load($name, $db);
-        } elseif (static::existsWithName($name, $db)) {
-            throw new DuplicateKeyException(
-                'Import Source %s already exists',
-                $name
-            );
-        } else {
-            $object = static::create([], $db);
-        }
-
-        $object->setProperties($properties);
-        if ($id !== null) {
-            // TODO: really?
-            $object->reallySet('id', $id);
-        }
-
-        return $object;
-    }
-
     public function setModifiers(array $modifiers)
     {
         if ($this->loadedRowModifiers === null && $this->hasBeenLoadedFromDb()) {
             $this->loadedRowModifiers = $this->fetchRowModifiers();
         }
-        $current = $this->loadedRowModifiers;
-        if ($current !== null && count($current) !== count($modifiers)) {
+        $current = (array) $this->loadedRowModifiers;
+        if (\count($current) !== \count($modifiers)) {
             $this->newRowModifiers = $modifiers;
         } else {
             $i = 0;
@@ -129,9 +70,9 @@ class ImportSource extends DbObjectWithSettings implements ExportInterface
                 if ($this->loadedRowModifiers[$i]->hasBeenModified()) {
                     $modified = true;
                 }
+                $i++;
             }
             if ($modified) {
-                // TOOD: no newRowModifiers, directly store loaded ones if diff
                 $this->newRowModifiers = $modifiers;
             }
         }
@@ -203,6 +144,10 @@ class ImportSource extends DbObjectWithSettings implements ExportInterface
         );
     }
 
+    /**
+     * @deprecated please use \Icinga\Module\Director\Data\FieldReferenceLoader
+     * @return array
+     */
     protected function exportRowModifiers()
     {
         $modifiers = [];
@@ -299,7 +244,7 @@ class ImportSource extends DbObjectWithSettings implements ExportInterface
         return null;
     }
 
-    public function applyModifiers(& $data)
+    public function applyModifiers(&$data)
     {
         $modifiers = $this->fetchFlatRowModifiers();
 
@@ -307,22 +252,42 @@ class ImportSource extends DbObjectWithSettings implements ExportInterface
             return $this;
         }
 
-
         foreach ($modifiers as $modPair) {
             /** @var PropertyModifierHook $modifier */
-            list($property, $modifier) = $modPair;
+            /** @var ?Filter $filter */
+            list($property, $modifier, $filter) = $modPair;
             $rejected = [];
+            $newRows = [];
             foreach ($data as $key => $row) {
+                if ($filter && ! $filter->matches($row)) {
+                    continue;
+                }
                 $this->applyPropertyModifierToRow($modifier, $property, $row);
                 if ($modifier->rejectsRow()) {
                     $rejected[] = $key;
                     $modifier->rejectRow(false);
                 }
+                if ($modifier->expandsRows()) {
+                    $target = $modifier->getTargetProperty($property);
+
+                    $newValue = $row->$target;
+                    if (\is_array($newValue)) {
+                        foreach ($newValue as $val) {
+                            $newRow = clone $row;
+                            $newRow->$target = $val;
+                            $newRows[] = $newRow;
+                        }
+                        $rejected[] = $key;
+                    }
+                }
             }
 
             foreach ($rejected as $key) {
                 unset($data[$key]);
             }
+            foreach ($newRows as $row) {
+                $data[] = $row;
+            }
         }
 
         return $this;
@@ -413,7 +378,12 @@ class ImportSource extends DbObjectWithSettings implements ExportInterface
     {
         $mods = [];
         foreach ($this->fetchRowModifiers() as $mod) {
-            $mods[] = [$mod->get('property_name'), $mod->getInstance()];
+            if ($filterExpression = $mod->get('filter_expression')) {
+                $filter = FilterEnrichment::enrichFilter(Filter::fromQueryString($filterExpression));
+            } else {
+                $filter = null;
+            }
+            $mods[] = [$mod->get('property_name'), $mod->getInstance(), $filter];
         }
 
         return $mods;
@@ -509,7 +479,7 @@ class ImportSource extends DbObjectWithSettings implements ExportInterface
     protected function raiseLimits()
     {
         MemoryLimit::raiseTo('1024M');
-        ini_set('max_execution_time', 0);
+        ini_set('max_execution_time', '0');
 
         return $this;
     }
