@@ -29,6 +29,10 @@ class PropertyModifierBitmask extends PropertyModifierHook
 
     public function transform($value)
     {
+        if ($value === null) {
+            return null;
+        }
+
         $mask = (int) $this->getSetting('bitmask');
         return (((int) $value) & $mask) === $mask;
     }
